/*
 *  Settings.cs
 *  Copyright (C) 2005-2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Collections;
using System.Xml;
using System.IO;


namespace Incollector.Engine {

	public struct Option {
		public string key;
		public string val;
	}
	


	/// <summary>Settings database class</summary>
	public static class Settings {
		private static ArrayList Data;
		private static bool      saving_now;
		
		
		/// <summary>Loads settings database</summary>
		public static bool Load ()
		{
			Data = new ArrayList ();
			
			StreamReader reader;
			
			try {
				reader = new StreamReader (FilesPaths.SettingsFile, System.Text.Encoding.UTF8);
			} catch  {
				LoadDefaultSettings ();
				return false;
			}
			
			XmlTextReader read = new XmlTextReader (reader);
			Option        opt  = new Option ();
			
			try {
				while (read.Read ())
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement))
						if ((read.Name == "option") && (read.NodeType == XmlNodeType.Element)) {
							opt     = new Option ();
							opt.key = read.GetAttribute ("key");
							opt.val = read.ReadString ();

							Data.Add (opt);
						}
			} catch {
				LoadDefaultSettings ();
				reader.Close ();
				return false;
			}
			

			reader.Close ();
			return true;
		}


		/// <summary>Saves settings database</summary>
		public static void Save ()
		{
			if (saving_now == true)
				return;
			
			saving_now = true;
			
			// save settings
			XmlTextWriter f = new XmlTextWriter (FilesPaths.SettingsFile + ".saving", System.Text.Encoding.UTF8);

			try {
				f.Formatting = Formatting.Indented;
				f.WriteStartDocument ();
				f.WriteStartElement (null, "settings", "");
					foreach (Option opt in Data) {
						f.WriteStartElement ("option");
						f.WriteAttributeString ("key", opt.key);
							f.WriteString (opt.val);
						f.WriteEndElement ();
					}
				f.WriteEndElement ();
				
				f.Close ();
			} finally {
				try {
					File.Delete(FilesPaths.SettingsFile);
				} finally {
					File.Move(FilesPaths.SettingsSavingFile, FilesPaths.SettingsFile);
				}
			}
			
			saving_now = false;
		}
		
		
		/// <summary>Returns string value</summary>
		/// <param name="key">Key whose value is returned</param>
		public static string Get (string key)
		{
			foreach (Option opt in Data) {
				if (opt.key == key)
					return opt.val;
			}

			return "";
		}
		
		
		/// <summary>Returns int value</summary>
		/// <param name="key">Key whose value is returned</param>
		/// <remarks>If key doesn't exist, function returns -1 value.</remarks>
		public static int GetInt (string key)
		{
			int rtn = -1;
			
			try {
				rtn = System.Convert.ToInt32 (Get (key));
			} catch {
				return -1;
			}
			
			return rtn;
		}
		
		
		/// <summary>Returns bool value</summary>
		/// <param name="key">Key whose value is returned</param>
		/// <remarks>If key doesn't exist, function returns true value.</remarks>
		public static bool GetBool (string key)
		{
			if (Get (key) == "")
				return true;			
			
			return System.Convert.ToBoolean (Get (key));
		}
		
		
		/// <summary>Returns bool value</summary>
		/// <param name="key">Key whose value is returned</param>
		/// <remarks>If key doesn't exist, function returns false value.</remarks>
		public static bool GetBoolDF (string key)
		{
			if (Get (key) == "")
				return false;			
			
			return System.Convert.ToBoolean (Get (key));
		}


		/// <summary>Returns Gdk.Color value</summary>
		/// <param name="key">Key whose value is returnd</param>
		public static Gdk.Color GetColor (string key)
		{
			Gdk.Color clr = new Gdk.Color (255, 255, 255);
			Gdk.Color.Parse (Engine.Settings.Get (key), ref clr);
			
			return clr;
		}
		
		
		/// <summary>Return strings list</summary>
		/// <param name="key">Key whose value is returnd</param>
		public static ArrayList GetList (string key)
		{
			ArrayList list = new ArrayList ();
			foreach (string item in Engine.Settings.Get (key).Split (';'))
				if (item != "")
					list.Add (item);
			
			return list;
		}
		
		
		/// <summary>Sets value of the "key" as "val"</summary>
		/// <param name="key">Key whose value is changed to "val"</param>
		/// <remarks>Function adds key to the database, if it doesn't exist</remarks>
		public static void Set (string key, string val)
		{
			int i = 0;
			
			foreach (Option opt in Data) {
				if (opt.key == key) {
					Option newopt = new Option ();
					newopt.key = key;
					newopt.val = val;
					Data [i] = newopt;
					return;
				}

				i++;
			}


			Option nopt = new Option ();
			nopt.key    = key;
			nopt.val    = val;
			
			Data.Add (nopt);

			return;
		}
		
		
		/// <summary>Sets int value of the "key" as "val"</summary>
		/// <param name="key">Key whose value is changed to "val"</param>
		public static void SetInt (string key, int val)
		{
			Set (key, val.ToString ());
		}
		
		
		/// <summary>Sets bool value of the "key" as "val"</summary>
		/// <param name="key">Key whose value is changed to "val"</param>
		public static void SetBool (string key, bool val)
		{
			Set (key, val.ToString ());
		}



		/// <summary>Sets Gdk.Color value of the "key" as "val"</summary>
		/// <param name="key">Key whose value is changed to "val"</param>
		public static void SetColor (string key, Gdk.Color clr)
		{
			try {
				string [] col = clr.ToString ().Split (':');
				string [] crs = col [1].Split ('/');
			
				Set (key, String.Format ("#{0}{1}{2}", ColorNormal (crs [0]), ColorNormal (crs [1]), ColorNormal (crs [2])));
			} catch {
			}
		}


		private static string ColorNormal (string clr)
		{
			if (clr.Length == 1)
				return "0" + clr;
			else
				return clr.Substring (0, 2);
		}
		
		
		/// <summary>Sets list value of the "key" as "list".</summary>
		/// <param name="key">Key whose value is changed to "list"</param>
		public static void SetList (string key, ArrayList list)
		{
			string NewList = "";
			
			foreach (string item in list)
				NewList += item + ";";
			
			Set (key, NewList);
		}

		
		/// <summary>Returns true if key exists</summary>
		/// <param name="key">Key to check</param>
		public static bool KeyExists (string key)
		{
			foreach (Option opt in Data)
				if (opt.key == key)
					return true;
			
			return false;
		}
		
		
		/// <summary>Loads default settings</summary>
		/// <remarks>Using when the settings file doesn't exist</remarks>
		public static void LoadDefaultSettings ()
		{
			SetBool ("DateFirstYear", true);
			SetBool ("Time24Mode", false);
			SetBool ("MinimizeToTray", true);
			SetBool ("ShowOnlyTrayIcon", false);
			SetBool ("UseDefaultColors", false);
			SetBool ("ShowPreview", false);
			SetBool ("ImportDates", false);
			SetBool ("ImporRating", false);
			SetBool ("ImportTags", true);
			
			SetColor ("SidebarColor", new Gdk.Color (237, 237, 237));
			SetColor ("PreviewColor", new Gdk.Color (255, 255, 255));
			SetColor ("SearchPanelColor", new Gdk.Color (230, 230, 230));
			SetColor ("ImportedEntriesPanelColor", new Gdk.Color (230, 230, 230));
		}
	}
}
